// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account_wrapper.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

abstract class AuthenticationRepository<T> extends BaseRepository {
  FormRepository get formRepository;

  FutureResult<Account> signUp({
    required String email,
    required String password,
  });

  FutureResult<Account> signInWithEmailAndPassword({
    required String email,
    required String password,
  });

  FutureResult<void> signOut();

  Stream<FutureResult<AccountWrapper<T>>> streamAccount();

  FutureResult<String> getIdentityToken();

  FutureResult<Account> getAccount();
  FutureResult<void> setAccount(Account account);

  FutureResult<T> getData();
  FutureResult<void> setData(T? data);

  FutureResult<AccountWrapper<T>> getCache();
  FutureResult<void> destroyCache();
}
